@@ -1,3 +1,4 @@
+from jsonschema import Draft4Validator
 import logging
 import sys
 import re
@@ -124,6 +125,18 @@ def _verify_format(s, format):
     return False
 
 
+def _verify_schema(data, schema):
+    validator = Draft4Validator(schema)
+    validator.iter_errors(data)
+    errors = [e.message for e in validator.iter_errors(data)]
+    if errors:
+        logging.error(
+            f"Error validating package contents {data} against schema {schema}. Details: {errors}"
+        )
+        return False
+    return True
+
+
 def _standardize_str(s):
     """
     Remove unnecessary spaces
